Package: DoseFinding
Type: Package
Title: Planning and Analyzing Dose Finding Experiments
Version: 1.4-1
Date: 2025-07-08
Authors@R: c(
   person("Bjoern", "Bornkamp", email = "bjoern.bornkamp@novartis.com", 
   comment = c(ORCID = "0000-0002-6294-8185"), role = c("aut")),
   person("Jose", "Pinheiro", role = "aut"),
   person("Frank", "Bretz", role = "aut"),
   person("Ludger", "Sandig", role = "aut"),
   person("Marius", "Thomas", role = c("aut", "cre"), email = "marius.thomas@novartis.com"),
   person("Daniel", "Sabanes Bove", role = "aut", email = "daniel.sabanes_bove@rconis.com",
           comment = c(ORCID = "0000-0002-0176-9239")),
   person(given = "Novartis Pharma AG", role = c("cph", "fnd")))
Imports: ggplot2, lattice, mvtnorm
Suggests: emmeans, numDeriv, Rsolnp, quadprog, parallel, mmrm,
        multcomp, knitr, rmarkdown, MASS, testthat (>= 3.0.0), tibble,
        RBesT, nlme, dplyr, tidyr
Maintainer: Marius Thomas <marius.thomas@novartis.com>
Description: The DoseFinding package provides functions for the design and analysis
	     of dose-finding experiments (with focus on pharmaceutical Phase
	     II clinical trials). It provides functions for: multiple contrast
	     tests, fitting non-linear dose-response models (using Bayesian and
	     non-Bayesian estimation), calculating optimal designs and an
	     implementation of the MCPMod methodology (Pinheiro et al. (2014)
	     <doi:10.1002/sim.6052>).
VignetteBuilder: knitr
License: GPL-3
LazyLoad: yes
RoxygenNote: 7.3.2
Encoding: UTF-8
URL: https://github.com/openpharma/DoseFinding,
        https://openpharma.github.io/DoseFinding/
BugReports: https://github.com/openpharma/DoseFinding/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-07-09 15:57:59 UTC; THOMAM1D
Author: Bjoern Bornkamp [aut] (ORCID: <https://orcid.org/0000-0002-6294-8185>),
  Jose Pinheiro [aut],
  Frank Bretz [aut],
  Ludger Sandig [aut],
  Marius Thomas [aut, cre],
  Daniel Sabanes Bove [aut] (ORCID:
    <https://orcid.org/0000-0002-0176-9239>),
  Novartis Pharma AG [cph, fnd]
Repository: CRAN
Date/Publication: 2025-07-09 18:00:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:55:29 UTC; unix
Archs: DoseFinding.so.dSYM
